@@ -16,12 +16,14 @@ use Ref::Util;
 use HTML::Show;
 use UUID::Tiny ':std';
 
-our $VERSION = '0.013';    # VERSION
+our $VERSION = '0.014';    # VERSION
 
 # ABSTRACT: Generate html/javascript charts from perl data using javascript library plotly.js
 
 sub render_full_html {
+    ## no critic
     my %params = validate( @_, { data => { type => ARRAYREF | OBJECT }, } );
+    ## use critic
 
     my $data     = $params{'data'};
     my $chart_id = create_uuid_as_string(UUID_TIME);
@@ -59,7 +61,7 @@ sub _render_cell {
     }
     my $template = <<'TEMPLATE';
 <div id="{$chart_id}"></div>
-<script src="https://cdn.plot.ly/plotly-1.21.2.min.js"></script>
+<script src="https://cdn.plot.ly/plotly-1.35.2.min.js"></script>
 <script>
 Plotly.plot(document.getElementById('{$chart_id}'),{$data} {$layout});
 </script>
@@ -124,7 +126,7 @@ Chart::Plotly - Generate html/javascript charts from perl data using javascript
 
 =head1 VERSION
 
-version 0.013
+version 0.014
 
 =head1 SYNOPSIS
 
@@ -138,16 +140,14 @@ version 0.013
  
  show_plot([$data]);
  
- use HTML::Show;
  use aliased 'Chart::Plotly::Trace::Scattergl';
  
  my $big_array = [ 1 .. 10000 ];
  my $scattergl = Scattergl->new( x => $big_array, y => [ map { rand 100 } @$big_array ] );
  
- HTML::Show::show( Chart::Plotly::render_full_html( data => [$scattergl] ) );
+ show_plot([$scattergl]);
 
- use HTML::Show;
- use Chart::Plotly;
+ use Chart::Plotly qw(show_plot);
  use PDL;
  
  use aliased 'Chart::Plotly::Trace::Surface';
@@ -159,7 +159,7 @@ version 0.013
  
  my $surface = Surface->new( x => $x, y => $y, z => $z );
  
- HTML::Show::show( Chart::Plotly::render_full_html( data => [$surface] ) );
+ show_plot([$surface]);
  
  use PDL::Math;
  
@@ -170,7 +170,7 @@ version 0.013
      z => bessj0( rvals( zeroes( $bessel_size, $bessel_size ) ) / 2 )
  );
  
- HTML::Show::show( Chart::Plotly::render_full_html( data => [$bessel] ) );
+ show_plot([$bessel]);
 
 =head1 DESCRIPTION
 
@@ -249,7 +249,7 @@ Pablo Rodríguez González <pablo.rodriguez.gonzalez@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2017 by Pablo Rodríguez González.
+This software is Copyright (c) 2017 by Pablo Rodríguez González.
 
 This is free software, licensed under:
 
